# Copyright (c) 2020-2021. Uniontech Software Ltd. All rights reserved.
#
# Author:     KeZhihuan <kezhihuan@uniontech.com>
#
# Maintainer: KeZhihuan <kezhihuan@uniontech.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import os
import shutil
import configparser
from buildstream import Element, ElementError, Scope, utils

class BundleBuildElement(Element):

    BST_STRICT_REBUILD = True

    def configure(self, node):
        self.node_validate(node, [
            'directory', 'include', 'exclude'
        ])
        self.directory = self.node_subst_member(node, 'directory')
        self.include = self.node_get_member(node, list, 'include')
        self.exclude = self.node_get_member(node, list, 'exclude')
        self.metadata = configparser.ConfigParser()

    def preflight(self):
        runtime_deps = list(self.dependencies(Scope.RUN, recurse=False))
        if runtime_deps:
            raise ElementError("{}: only build type dependencies supported by bundle_build elements"
                               .format(self))

        sources = list(self.sources())
        if sources:
            raise ElementError("{}: bundle_build elements may not have sources".format(self))

    def get_unique_key(self):
        key = {}
        key['directory'] = self.directory
        key['include'] = sorted(self.include)
        key['exclude'] = sorted(self.exclude)
        return key

    def configure_sandbox(self, sandbox):
        pass

    def stage(self, sandbox):
        pass

    def assemble(self, sandbox):
        self.stage_sources(sandbox, 'input')
        base_dir = sandbox.get_directory()
        all_files = os.path.join(base_dir, 'buildstream', 'allfiles/usr')
        more_files = os.path.join(base_dir, 'buildstream', 'allfiles')
        share_files = os.path.join(all_files, 'share')
        rel_directory = os.path.relpath(self.directory, '/')
        sub_dir = os.path.join(all_files, rel_directory)
        share_dir = os.path.join(share_files, rel_directory)
        loader_files = os.path.join(more_files, 'loader')
        info_files = os.path.join(more_files, 'info.json')
        files_files = os.path.join(more_files, 'files')
        entries_files = os.path.join(more_files, 'entries')
        install_dir = os.path.join(base_dir, 'buildstream', 'install')
        files_dir = os.path.join(install_dir, 'files')
        entries_dir = os.path.join(install_dir, 'entries')
        output_dir = os.path.join(install_dir, 'outputs')
        opshare_dir = os.path.join(output_dir, 'share')
        applications_dir = os.path.join(opshare_dir, 'applications')
        loader_dir = os.path.join(install_dir, 'loader')
        info_dir = os.path.join(install_dir, 'info.json')
        stage_dir = os.path.join(os.sep, 'buildstream', 'allfiles')

        os.makedirs(all_files, exist_ok=True)
        os.makedirs(files_dir, exist_ok=True)
        os.makedirs(output_dir, exist_ok=True)
        os.makedirs(opshare_dir, exist_ok=True)

        with self.timed_activity("creating bundle build", silent_nested=True):
            self.stage_dependency_artifacts(sandbox, Scope.BUILD,
                                            path=stage_dir,
                                            include=self.include,  
                                            exclude=self.exclude)
            if os.path.exists(files_files):
                utils.link_files(files_files, files_dir)
            else:
                utils.link_files(sub_dir, files_dir)
            if os.path.exists(entries_files):
                utils.link_files(entries_files,entries_dir)
            else:
                os.makedirs(entries_dir, exist_ok=True)
                utils.link_files(share_dir, entries_dir)
            os.link(info_files,info_dir)
            os.link(loader_files, loader_dir)
            utils.link_files(entries_dir,opshare_dir)
            shutil.rmtree(applications_dir)
            os.makedirs(applications_dir, exist_ok=True)
            for cur_dir,dirs,files in os.walk(more_files):
                for filesName in files:
                    if filesName.endswith('.desktop'):
                        desktop_files = os.path.join(more_files,filesName)
                        desktopName = filesName
            desktop_dir = os.path.join(applications_dir, desktopName)
            os.link(desktop_files,desktop_dir)
        return os.path.join(os.sep, 'buildstream', 'install')

def setup():
    return BundleBuildElement
